@@ -12,7 +12,7 @@ class tanf(Variable):
     documentation = (
         "Amount of Temporary Assistance for Needy Families benefit received."
     )
-    unit = "currency-USD"
+    unit = USD
 
     def formula(spm_unit, period, parameters):
         # Obtain eligibility.
@@ -22,6 +22,45 @@ class tanf(Variable):
         return where(eligible, amount_if_eligible, 0)
 
 
+class continuous_tanf_eligibility(Variable):
+    value_type = bool
+    entity = SPMUnit
+    definition_period = YEAR
+    label = "Continued Economic Eligibility for TANF"
+    documentation = "Whether the familiy meets the economic requirements for the Temporary Assistance for Needy Families program after being approved."
+
+
+class initial_tanf_eligibility(Variable):
+    value_type = bool
+    entity = SPMUnit
+    definition_period = YEAR
+    label = "Initial Economic Eligibility for TANF"
+    documentation = "Whether the familiy meets the economic requirements for the Temporary Assistance for Needy Families program on application."
+
+
+class is_person_demographic_tanf_eligible(Variable):
+    value_type = bool
+    entity = Person
+    definition_period = YEAR
+    label = "Person-level eligiblity for TANF based on age, pregnancy, etc."
+    documentation = "Whether a person in a family applying for the Temporary Assistance for Needy Families program meets demographic requirements."
+
+    def formula(person, period, parameters):
+        child_0_17 = person("is_child", period)
+        is_18 = person("age", period) == 18
+        school_enrolled_18_year_old = person("is_in_school", period) & is_18
+        pregnant = person("is_pregnant", period)
+        return child_0_17 | school_enrolled_18_year_old | pregnant
+
+
+class is_tanf_enrolled(Variable):
+    value_type = bool
+    entity = SPMUnit
+    definition_period = YEAR
+    label = "Current Enrollement in TANF"
+    documentation = "Whether the familiy is currently enrolled in the Temporary Assistance for Needy Families program."
+
+
 class is_tanf_eligible(Variable):
     value_type = bool
     entity = SPMUnit
@@ -30,15 +69,15 @@ class is_tanf_eligible(Variable):
     documentation = "Whether the family is eligible for Temporary Assistance for Needy Families benefit."
 
     def formula(spm_unit, period, parameters):
-        children_0_17 = spm_unit.sum(spm_unit.members("is_child", period))
-        school_enrolled_18_year_olds = spm_unit.sum(
-            (spm_unit.members("is_in_school", period))
-            & (spm_unit.members("age", period) == 18)
+        demographic_eligible = spm_unit.any(
+            spm_unit.members("is_person_demographic_tanf_eligible", period)
         )
-        pregnant_people = spm_unit.sum(spm_unit.members("is_pregnant", period))
-        return (
-            children_0_17 + school_enrolled_18_year_olds + pregnant_people
-        ) > 0
+        economic_eligible = where(
+            spm_unit("is_tanf_enrolled", period),
+            spm_unit("continuous_tanf_eligibility", period),
+            spm_unit("initial_tanf_eligibility", period),
+        )
+        return demographic_eligible & economic_eligible
 
 
 # Quick fix, should be fixed by resolving https://github.com/openfisca/openfisca-core/issues/1085
@@ -57,7 +96,7 @@ class tanf_max_amount(Variable):
     definition_period = YEAR
     label = "TANF maximum benefit"
     documentation = "The maximum benefit amount a family could receive from Temporary Assistance for Needy Families given their state and family size."
-    unit = "currency-USD"
+    unit = USD
 
     def formula(spm_unit, period, parameters):
         family_size = spm_unit.nb_persons().astype(str)
@@ -72,7 +111,7 @@ class tanf_countable_income(Variable):
     definition_period = YEAR
     label = "TANF countable income"
     documentation = "Countable income for calculating Temporary Assistance for Needy Families benefit."
-    unit = "currency-USD"
+    unit = USD
 
     def formula(spm_unit, period, parameters):
         tanf_gross_income = spm_unit("tanf_total_gross_income", period)
@@ -89,7 +128,7 @@ class tanf_total_gross_income(Variable):
     definition_period = YEAR
     label = "TANF gross income"
     documentation = "Gross income for calculating Temporary Assistance for Needy Families benefit. Includes both gross earned and unearned income."
-    unit = "currency-USD"
+    unit = USD
     reference = "https://www.dhs.state.il.us/page.aspx?item=15814"
 
 
@@ -99,7 +138,7 @@ class tanf_amount_if_eligible(Variable):
     definition_period = YEAR
     label = "TANF amount if family is eligible"
     documentation = "How much a family would receive if they were eligible for Temporary Assistance for Needy Families benefit."
-    unit = "currency-USD"
+    unit = USD
 
     def formula(spm_unit, period, parameters):
         max_amount = spm_unit("tanf_max_amount", period)
